from abc import ABC, abstractmethod
from typing import Optional
from uuid import UUID

import lqs.interface.base.models as models


class CreateInterface(ABC):
    @abstractmethod
    def _api_key(self, **kwargs) -> models.APIKeyDataResponse:
        pass

    def api_key(
        self, user_id: UUID, name: str, disabled: bool = False
    ) -> models.APIKeyDataResponse:
        return self._api_key(
            user_id=user_id,
            name=name,
            disabled=disabled,
        )

    def _api_key_by_model(
        self, data: models.APIKeyCreateRequest
    ) -> models.APIKeyDataResponse:
        return self.api_key(**data.model_dump())

    @abstractmethod
    def _role(self, **kwargs) -> models.RoleDataResponse:
        pass

    def role(
        self,
        name: str,
        policy: dict,
        note: Optional[str] = None,
        disabled: Optional[bool] = False,
        managed: Optional[bool] = False,
        default: Optional[bool] = False,
    ) -> models.RoleDataResponse:
        return self._role(
            name=name,
            policy=policy,
            note=note,
            disabled=disabled,
            managed=managed,
            default=default,
        )

    def _role_by_model(self, data: models.RoleCreateRequest) -> models.RoleDataResponse:
        return self.role(**data.model_dump())

    @abstractmethod
    def _user(self, **kwargs) -> models.UserDataResponse:
        pass

    def user(
        self,
        username: str,
        role_id: Optional[UUID] = None,
        admin: Optional[bool] = False,
        disabled: Optional[bool] = False,
        managed: Optional[bool] = False,
        external_id: Optional[str] = None,
        password: Optional[str] = None,
    ) -> models.UserDataResponse:
        return self._user(
            username=username,
            role_id=role_id,
            admin=admin,
            disabled=disabled,
            managed=managed,
            external_id=external_id,
            password=password,
        )

    def _user_by_model(self, data: models.UserCreateRequest) -> models.UserDataResponse:
        return self.user(**data.model_dump())



from typing import List, Optional
from datetime import datetime

from pydantic import BaseModel

from lqs.interface.base.models.__common__ import (
    ResourceModel,
    UploadState,
)


class Object(ResourceModel["Object"]):
    _repr_fields = ("key",)

    key: str
    etag: Optional[str]
    size: Optional[int]
    last_modified: Optional[datetime]
    presigned_url: Optional[str]
    upload_state: UploadState


class ObjectDataResponse(BaseModel):
    data: Object


class ObjectListResponse(BaseModel):
    data: List[Object]
    is_truncated: Optional[bool]
    key_count: Optional[int]
    max_keys: int

    continuation_token: Optional[str]
    next_continuation_token: Optional[str]
    prefix: Optional[str]
    start_after: Optional[str]
    delimiter: Optional[str]
    common_prefixes: Optional[List[str]]


class ObjectCreateRequest(BaseModel):
    key: str
    content_type: Optional[str] = None


class ObjectUpdateRequest(BaseModel):
    upload_state: UploadState


# Object Parts


class ObjectPart(BaseModel):
    part_number: int
    etag: str
    size: int
    last_modified: Optional[datetime]
    presigned_url: Optional[str]


class ObjectPartDataResponse(BaseModel):
    data: ObjectPart


class ObjectPartListResponse(BaseModel):
    data: List[ObjectPart]
    part_number_marker: Optional[int]
    next_part_number_marker: Optional[int]
    max_parts: Optional[int]
    is_truncated: Optional[bool]


class ObjectPartCreateRequest(BaseModel):
    part_number: Optional[int] = None
    size: int



from typing import List, Optional, Tuple
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    PaginationModel,
    ProcessModel,
    ProcessCreateRequest,
    ProcessUpdateRequest,
    optional_field,
    Int64,
)


class DigestionPartIndexEntry(BaseModel):
    topic_id: str
    ingestion_id: Optional[str]
    source: Optional[str]

    data_offset: int
    data_length: int

    chunk_compression: Optional[str]
    chunk_offset: Optional[int]
    chunk_length: Optional[int]
    timestamp: Int64


# TODO: in Python 3.11, we can use Tuple[*get_type_hints(DigestionPartIndex).values()]
DigestionPartIndexTuple = Tuple[
    str,
    Optional[str],
    Optional[str],
    int,
    int,
    Optional[str],
    Optional[int],
    Optional[int],
    Int64,
]


class DigestionPartIndex(BaseModel):
    digestion_part_id: UUID
    index: Optional[List[DigestionPartIndexTuple]]


class DigestionPartIndexCreateRequest(BaseModel):
    digestion_part_id: UUID
    index: Optional[List[DigestionPartIndexTuple]] = None


class DigestionPartIndexUpdateRequest(BaseModel):
    index: Optional[List[DigestionPartIndexTuple]] = optional_field


class DigestionPartIndexDataResponse(DataResponseModel[DigestionPartIndex]):
    pass


class DigestionPartIndexListResponse(PaginationModel[DigestionPartIndex]):
    pass


# Digestion Part


class DigestionPart(ProcessModel["DigestionPart"]):
    sequence: int
    digestion_id: UUID

    index: Optional[List[DigestionPartIndexTuple]]

    log_id: Optional[UUID]
    group_id: Optional[UUID]


class DigestionPartCreateRequest(ProcessCreateRequest):
    sequence: int
    index: Optional[List[DigestionPartIndexTuple]] = None


class DigestionPartUpdateRequest(ProcessUpdateRequest):
    sequence: int = optional_field
    index: Optional[List[DigestionPartIndexTuple]] = optional_field


class DigestionPartDataResponse(DataResponseModel[DigestionPart]):
    pass


class DigestionPartListResponse(PaginationModel[DigestionPart]):
    pass



from typing import List, Optional
from uuid import UUID

from pydantic import BaseModel


class Command(BaseModel):
    action: str
    resource: Optional[str]
    app: Optional[str]
    datastore_id: Optional[UUID]
    kwargs: Optional[dict]
    process_states: Optional[List[str]]
    process_types: Optional[List[str]]


class CommandCreateRequest(BaseModel):
    action: str
    resource: Optional[str] = None
    app: Optional[str] = None
    datastore_id: Optional[UUID] = None
    kwargs: Optional[dict] = None
    process_states: Optional[List[str]] = None
    process_types: Optional[List[str]] = None


class CommandDataResponse(BaseModel):
    data: Optional[dict] = None



from enum import Enum

from lqs.interface.base.models.__common__ import (  # noqa: F401
    CommonModel,
    DataResponseModel,
    TimeSeriesModel,
    PaginationModel,
    PatchOperation,
    JSONFilter,
    optional_field,
    Int64,
)


class ProcessState(str, Enum):
    ready = "ready"
    queued = "queued"
    processing = "processing"
    finalizing = "finalizing"
    completed = "completed"
    failed = "failed"
    cancelled = "cancelled"
    archived = "archived"


class ProcessType(str, Enum):
    ingestion = "ingestion"
    ingestion_part = "ingestion_part"
    digestion = "digestion"
    digestion_part = "digestion_part"


class JobType(str, Enum):
    ingestion = "ingestion"
    post_ingestion = "post_ingestion"
    failed_ingestion = "failed_ingestion"

    ingestion_part = "ingestion_part"
    post_ingestion_part = "post_ingestion_part"
    failed_ingestion_part = "failed_ingestion_part"

    extraction = "extraction"
    post_extraction = "post_extraction"
    failed_extraction = "failed_extraction"

    inference = "inference"
    post_inference = "post_inference"
    failed_inference = "failed_inference"

    copy = "copy"
    post_copy = "post_copy"
    failed_copy = "failed_copy"



from typing import List, Optional
from uuid import UUID

from pydantic import Field

from lqs.interface.core.models.__common__ import (
    CommonModel,
    DataResponseModel,
    LockModel,
    LockCreateRequest,
    LockUpdateRequest,
    PaginationModel,
    TypeEncoding,
    optional_field,
    Int64,
)
from lqs.interface.core.models import Record


class Topic(CommonModel, LockModel):
    log_id: UUID = Field(
        ..., description="The ID of the log to which this topic belongs."
    )
    group_id: Optional[UUID] = Field(
        ..., description="The ID of the group to which this topic belongs."
    )

    name: str = Field(..., description="The name of the topic (unique per log).")
    note: Optional[str] = Field(..., description="A general note about the topic.")
    context: Optional[dict] = Field(..., description="A JSON context for the topic.")

    associated_topic_id: Optional[UUID] = Field(
        ..., description="The ID of an associated topic (if any) for reference."
    )
    start_time: Optional[Int64] = Field(
        ..., description="The timestamp of the first record of the topic."
    )
    end_time: Optional[Int64] = Field(
        ..., description="The timestamp of the last record of the topic."
    )

    record_size: int = Field(
        ..., description="The total size of all records in the topic in bytes."
    )
    record_count: int = Field(
        ..., description="The total number of records in the topic."
    )
    object_size: int = Field(
        ..., description="The total size of all objects in the topic in bytes."
    )
    object_count: int = Field(
        ..., description="The total number of objects in the topic."
    )

    strict: bool = Field(
        ..., description="Whether the topic's schema should be strictly enforced."
    )
    type_name: Optional[str] = Field(
        ...,
        description="The name of the message type which the topic's records should conform to.",
    )
    type_encoding: Optional[TypeEncoding] = Field(
        ..., description="The encoding of the message data of the topic's records."
    )
    type_data: Optional[str] = Field(
        ...,
        description="The definition of the message type used to (de)serialize the topic's records.",
    )
    type_schema: Optional[dict] = Field(
        ...,
        description="A JSON schema describing the record data of the topic's records.",
    )

    def list_records(self, **kwargs) -> List["Record"]:
        return self._app.list.record(topic_id=self.id, **kwargs).data


Topic.model_rebuild()


class TopicDataResponse(DataResponseModel[Topic]):
    pass


class TopicListResponse(PaginationModel[Topic]):
    pass


class TopicCreateRequest(LockCreateRequest):
    log_id: UUID
    name: str
    note: Optional[str] = None
    context: Optional[dict] = None
    associated_topic_id: Optional[UUID] = None

    strict: bool = False
    type_name: Optional[str] = None
    type_encoding: Optional[TypeEncoding] = None
    type_data: Optional[str] = None
    type_schema: Optional[dict] = None


class TopicUpdateRequest(LockUpdateRequest):
    name: str = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    associated_topic_id: Optional[UUID] = optional_field

    strict: bool = optional_field
    type_name: Optional[str] = optional_field
    type_encoding: Optional[TypeEncoding] = optional_field
    type_data: Optional[str] = optional_field
    type_schema: Optional[dict] = optional_field



from typing import Optional
from uuid import UUID

from pydantic import Field

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    PaginationModel,
    ProcessModel,
    ProcessCreateRequest,
    ProcessUpdateRequest,
    optional_field_alt as optional_field,
)


class Digestion(ProcessModel["Digestion"]):
    log_id: UUID = Field(
        ..., description="The ID of the log to which this digestion belongs."
    )
    group_id: Optional[UUID] = Field(
        ..., description="The ID of the group to which this digestion belongs."
    )
    name: Optional[str] = Field(
        ..., description="The name of the digestion (not unique)."
    )
    note: Optional[str] = Field(
        ..., description="A general note about the digestion for reference."
    )
    context: Optional[dict] = Field(
        ..., description="A JSON context for the digestion."
    )


class DigestionDataResponse(DataResponseModel[Digestion]):
    pass


class DigestionListResponse(PaginationModel[Digestion]):
    pass


class DigestionCreateRequest(ProcessCreateRequest):
    log_id: UUID = Field(
        ..., description="The ID of the log to which the digestion should be added."
    )
    name: Optional[str] = Field(
        None, description="The name of the digestion (not unique)."
    )
    note: Optional[str] = Field(
        None, description="A general note about the digestion for reference."
    )
    context: Optional[dict] = Field(
        None, description="A JSON context for the digestion."
    )


class DigestionUpdateRequest(ProcessUpdateRequest):
    name: Optional[str] = optional_field("The name of the digestion (not unique).")
    note: Optional[str] = optional_field(
        "A general note about the digestion for reference."
    )
    context: Optional[dict] = optional_field("A JSON context for the digestion.")



from abc import abstractmethod
from typing import Optional, Union
from uuid import UUID

from lqs.interface.base.fetch import FetchInterface as BaseFetchInterface
import lqs.interface.core.models as models


class FetchInterface(BaseFetchInterface):
    @abstractmethod
    def _digestion(self, **kwargs) -> models.DigestionDataResponse:
        pass

    def digestion(self, digestion_id: UUID):
        """
        Fetches a digestion by ID.

        Args:
            digestion_id: The ID of the digestion to fetch.
        Returns:
            A data response for the digestion.
        """
        return self._digestion(
            digestion_id=digestion_id,
        )

    @abstractmethod
    def _digestion_part(self, **kwargs) -> models.DigestionPartDataResponse:
        pass

    def digestion_part(
        self, digestion_part_id: UUID, digestion_id: Optional[UUID] = None
    ):
        """
        Fetches a digestion part by ID.

        Args:
            digestion_part_id: The ID of the digestion part to fetch.
            digestion_id: The ID of the digestion to which the digestion part belongs.
        Returns:
            A data response for the digestion part.
        """
        return self._digestion_part(
            digestion_id=digestion_id,
            digestion_part_id=digestion_part_id,
        )

    @abstractmethod
    def _digestion_topic(self, **kwargs) -> models.DigestionTopicDataResponse:
        pass

    def digestion_topic(
        self, digestion_topic_id: UUID, digestion_id: Optional[UUID] = None
    ):
        """
        Fetches a digestion topic by ID.

        Args:
            digestion_topic_id: The ID of the digestion topic to fetch.
            digestion_id: The ID of the digestion to which the digestion topic belongs.
        Returns:
            A data response for the digestion topic.
        """
        return self._digestion_topic(
            digestion_id=digestion_id,
            digestion_topic_id=digestion_topic_id,
        )

    @abstractmethod
    def _group(self, **kwargs) -> models.GroupDataResponse:
        pass

    def group(self, group_id: UUID):
        """
        Fetches a group by ID.

        Args:
            group_id: The ID of the group to fetch.
        Returns:
            A data response for the group.
        """
        return self._group(
            group_id=group_id,
        )

    @abstractmethod
    def _hook(self, **kwargs) -> models.HookDataResponse:
        pass

    def hook(self, hook_id: UUID, workflow_id: Optional[UUID] = None):
        """
        Fetches a hook by ID.

        Args:
            hook_id: The ID of the hook to fetch.
            workflow_id: The ID of the workflow to which the hook belongs.
        Returns:
            A data response for the hook.
        """
        return self._hook(
            workflow_id=workflow_id,
            hook_id=hook_id,
        )

    @abstractmethod
    def _ingestion(self, **kwargs) -> models.IngestionDataResponse:
        pass

    def ingestion(self, ingestion_id: UUID):
        """
        Fetches an ingestion by ID.

        Args:
            ingestion_id: The ID of the ingestion to fetch.
        Returns:
            A data response for the ingestion.
        """
        return self._ingestion(
            ingestion_id=ingestion_id,
        )

    @abstractmethod
    def _ingestion_part(self, **kwargs) -> models.IngestionPartDataResponse:
        pass

    def ingestion_part(
        self, ingestion_part_id: UUID, ingestion_id: Optional[UUID] = None
    ):
        """
        Fetches an ingestion part by ID.

        Args:
            ingestion_part_id: The ID of the ingestion part to fetch.
            ingestion_id: The ID of the ingestion to which the ingestion part belongs.
        Returns:
            A data response for the ingestion part.
        """
        return self._ingestion_part(
            ingestion_id=ingestion_id,
            ingestion_part_id=ingestion_part_id,
        )

    @abstractmethod
    def _label(self, **kwargs) -> models.LabelDataResponse:
        pass

    def label(self, label_id: UUID):
        """
        Fetches a label by ID.

        Args:
            label_id: The ID of the label to fetch.
        Returns:
            A data response for the label.
        """
        return self._label(
            label_id=label_id,
        )

    @abstractmethod
    def _log(self, **kwargs) -> models.LogDataResponse:
        pass

    def log(self, log_id: UUID):
        """
        Fetches a log by ID.

        Args:
            log_id: The ID of the log to fetch.
        Returns:
            A data response for the log.
        """
        return self._log(
            log_id=log_id,
        )

    @abstractmethod
    def _log_object(self, **kwargs) -> Union[models.ObjectDataResponse, bytes]:
        pass

    def log_object(
        self,
        object_key: str,
        log_id: UUID,
        redirect: Optional[bool] = False,
        offset: Optional[int] = None,
        length: Optional[int] = None,
    ) -> Union[models.ObjectDataResponse, bytes]:
        """
        Fetches an object by key.

        Args:
            object_key: The key of the object to fetch.
            log_id: The ID of the log to which the object belongs.
            redirect: Whether to redirect to the object store or return the object directly. Defaults to False.
            offset: The offset from which to read the object.
            length: The length of the object to read.
        Returns:
            A data response for the object or the object itself as bytes if redirect is True.
        """
        return self._log_object(
            log_id=log_id,
            object_key=object_key,
            redirect=redirect,
            offset=offset,
            length=length,
        )

    @abstractmethod
    def _log_object_part(self, **kwargs) -> models.ObjectPartDataResponse:
        pass

    def log_object_part(self, object_key: str, part_number: int, log_id: UUID):
        """
        Fetches an object part by key and part number.

        Args:
            object_key: The key of the object to fetch.
            part_number: The part number of the object to fetch.
            log_id: The ID of the log to which the object belongs.
        Returns:
            A data response for the object part.
        """
        return self._log_object_part(
            object_key=object_key,
            part_number=part_number,
            log_id=log_id,
        )

    @abstractmethod
    def _object(self, **kwargs) -> Union[models.ObjectDataResponse, bytes]:
        pass

    def object(
        self,
        object_key: str,
        object_store_id: UUID,
        redirect: Optional[bool] = False,
        offset: Optional[int] = None,
        length: Optional[int] = None,
    ) -> Union[models.ObjectDataResponse, bytes]:
        """
        Fetches an object by key.

        Args:
            object_key: The key of the object to fetch.
            object_store_id: The ID of the object store to which the object belongs.
            redirect: Whether to redirect to the object store or return the object directly. Defaults to False.
            offset: The offset from which to read the object.
            length: The length of the object to read.
        Returns:
            A data response for the object or the object itself as bytes if redirect is True.
        """
        return self._object(
            object_store_id=object_store_id,
            object_key=object_key,
            redirect=redirect,
            offset=offset,
            length=length,
        )

    @abstractmethod
    def _object_part(self, **kwargs) -> models.ObjectPartDataResponse:
        pass

    def object_part(self, object_key: str, part_number: int, object_store_id: UUID):
        """
        Fetches an object part by key and part number.

        Args:
            object_key: The key of the object to fetch.
            part_number: The part number of the object to fetch.
            object_store_id: The ID of the object store to which the object belongs.
        Returns:
            A data response for the object part.
        """
        return self._object_part(
            object_key=object_key,
            part_number=part_number,
            object_store_id=object_store_id,
        )

    @abstractmethod
    def _object_store(self, **kwargs) -> models.ObjectStoreDataResponse:
        pass

    def object_store(self, object_store_id: UUID):
        """
        Fetches an object store by ID.

        Args:
            object_store_id: The ID of the object store to fetch.
        Returns:
            A data response for the object store.
        """
        return self._object_store(
            object_store_id=object_store_id,
        )

    @abstractmethod
    def _query(self, **kwargs) -> models.QueryDataResponse:
        pass

    def query(self, query_id: UUID, log_id: Optional[UUID] = None):
        """
        Fetches a query by ID.

        Args:
            query_id: The ID of the query to fetch.
            log_id: The ID of the log to which the query belongs.
        Returns:
            A data response for the query.
        """
        return self._query(
            log_id=log_id,
            query_id=query_id,
        )

    @abstractmethod
    def _record(self, **kwargs) -> models.RecordDataResponse:
        pass

    def record(
        self,
        timestamp: models.Int64,
        topic_id: UUID,
        include_auxiliary_data: bool = False,
        include_raw_data: bool = False,
        auxiliary_context: Optional[str] = None,
    ):
        """
        Fetches a record by timestamp and topic ID.

        Args:
            timestamp: The timestamp of the record to fetch.
            topic_id: The ID of the topic to which the record belongs.
            include_auxiliary_data: Whether to include auxiliary data in the record. Defaults to False.
            include_raw_data: Whether to include raw data in the record. Defaults to False.
            auxiliary_context: The context for the auxiliary data generation to include in the record. Defaults to None.
        Returns:
            A data response for the record.
        """
        return self._record(
            timestamp=timestamp,
            topic_id=topic_id,
            include_auxiliary_data=include_auxiliary_data,
            include_raw_data=include_raw_data,
            auxiliary_context=auxiliary_context,
        )

    @abstractmethod
    def _tag(self, **kwargs) -> models.TagDataResponse:
        pass

    def tag(self, tag_id: UUID, log_id: Optional[UUID] = None):
        """
        Fetches a tag by ID.

        Args:
            tag_id: The ID of the tag to fetch.
            log_id: The ID of the log to which the tag belongs.
        Returns:
            A data response for the tag.
        """
        return self._tag(
            log_id=log_id,
            tag_id=tag_id,
        )

    @abstractmethod
    def _topic(self, **kwargs) -> models.TopicDataResponse:
        pass

    def topic(self, topic_id: UUID):
        """
        Fetches a topic by ID.

        Args:
            topic_id: The ID of the topic to fetch.
        Returns:
            A data response for the topic.
        """
        return self._topic(
            topic_id=topic_id,
        )

    @abstractmethod
    def _workflow(self, **kwargs) -> models.WorkflowDataResponse:
        pass

    def workflow(self, workflow_id: UUID):
        """
        Fetches a workflow by ID.

        Args:
            workflow_id: The ID of the workflow to fetch.
        Returns:
            A data response for the workflow.
        """
        return self._workflow(
            workflow_id=workflow_id,
        )



from typing import Optional
from uuid import UUID

from pydantic import Field

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    TimeSeriesModel,
    PaginationModel,
    LockModel,
    LockCreateRequest,
    LockUpdateRequest,
    optional_field,
    Int64,
)


class Record(TimeSeriesModel["Record"], LockModel):
    _repr_fields = ("timestamp", "topic_id", "log_id")

    log_id: UUID = Field(
        ..., description="The ID of the log to which this record's topic belongs."
    )
    topic_id: UUID = Field(
        ..., description="The ID of the topic to which this record belongs."
    )
    timestamp: Int64 = Field(
        ..., description="The timestamp, in nanoseconds, of the record."
    )

    ingestion_id: Optional[UUID] = Field(
        ..., description="The ID of the ingestion which created this record, if any."
    )
    data_offset: Optional[int] = Field(
        ...,
        description="The offset, in bytes, of the record's message data in the log object.",
    )
    data_length: Optional[int] = Field(
        ...,
        description="The length, in bytes, of the record's message data in the log object.",
    )
    chunk_compression: Optional[str] = Field(
        ...,
        description="The compression algorithm used to compress the record's message data in the log object, if any.",
    )
    chunk_offset: Optional[int] = Field(
        ...,
        description="The offset, in bytes, of the record's message data in the log object's uncompressed data relative to the start of the uncompressed chunk, if compressed.",
    )
    chunk_length: Optional[int] = Field(
        ...,
        description="The length, in bytes, of the record's message data in the log object's uncompressed data, if compressed.",
    )
    source: Optional[str] = Field(
        ...,
        description="A relative path to the record's log object relative to the record's ingestion object, if any.",
    )
    error: Optional[dict] = Field(
        ...,
        description="The JSON payload of an error that occurred during record processing.",
    )

    query_data: Optional[dict | list] = Field(
        ...,
        description="A JSON data representation of the record's message data which is queryable.",
    )
    auxiliary_data: Optional[dict | list] = Field(
        ...,
        description="A JSON data representation of the record's auxiliary data which can be used for functional purposes.",
    )
    raw_data: Optional[str] = Field(
        ...,
        description="A string representation of the record's message data, presented as-is. This data will be base 64 encoded if needed.",
    )
    context: Optional[dict] = Field(..., description="A JSON context for the record.")
    note: Optional[str] = Field(
        ..., description="A general note about the record for reference."
    )

    def get_auxiliary_data(self):
        if self.auxiliary_data is None:
            record = self.fetch(
                self.timestamp, self.topic_id, include_auxiliary_data=True
            )
            self.auxiliary_data = record.auxiliary_data
        return self.auxiliary_data

    def load_auxiliary_data_image(self):
        return self._app.utils.load_auxiliary_data_image(self)

    def get_raw_data(self):
        if self.raw_data is None:
            record = self.fetch(self.timestamp, self.topic_id, include_raw_data=True)
            self.raw_data = record.raw_data
        return self.raw_data


class RecordDataResponse(DataResponseModel[Record]):
    pass


class RecordListResponse(PaginationModel[Record]):
    pass


class RecordCreateRequest(LockCreateRequest):
    timestamp: Int64
    note: Optional[str] = None
    context: Optional[dict] = None
    query_data: Optional[dict] = None
    auxiliary_data: Optional[dict] = None

    data_offset: Optional[int] = None
    data_length: Optional[int] = None
    chunk_compression: Optional[str] = None
    chunk_offset: Optional[int] = None
    chunk_length: Optional[int] = None
    source: Optional[str] = None


class RecordUpdateRequest(LockUpdateRequest):
    error: Optional[dict] = optional_field
    query_data: Optional[dict] = optional_field
    auxiliary_data: Optional[dict] = optional_field
    context: Optional[dict] = optional_field
    note: Optional[str] = optional_field



from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.core.models.__common__ import (
    DataResponseModel,
    CommonModel,
    PaginationModel,
    optional_field,
    Int64,
)


class DigestionTopic(CommonModel):
    digestion_id: UUID
    topic_id: UUID
    start_time: Optional[Int64]
    end_time: Optional[Int64]
    frequency: Optional[float]
    query_data_filter: Optional[dict]
    context_filter: Optional[dict]


class DigestionTopicDataResponse(DataResponseModel[DigestionTopic]):
    pass


class DigestionTopicListResponse(PaginationModel[DigestionTopic]):
    pass


class DigestionTopicCreateRequest(BaseModel):
    topic_id: UUID
    start_time: Optional[Int64]
    end_time: Optional[Int64]
    frequency: Optional[float] = None
    query_data_filter: Optional[dict] = None
    context_filter: Optional[dict] = None


class DigestionTopicUpdateRequest(BaseModel):
    start_time: Optional[Int64] = optional_field
    end_time: Optional[Int64] = optional_field
    frequency: Optional[float] = optional_field
    query_data_filter: Optional[dict] = optional_field
    context_filter: Optional[dict] = optional_field



from lqs.interface.core.create import CreateInterface
from lqs.interface.core.delete import DeleteInterface
from lqs.interface.core.fetch import FetchInterface
from lqs.interface.core.list import ListInterface
from lqs.interface.core.update import UpdateInterface



from abc import ABC, abstractmethod
from uuid import UUID
from typing import Optional

import lqs.interface.admin.models as models


class CreateInterface(ABC):
    @abstractmethod
    def _command(self, **kwargs) -> models.CommandDataResponse:
        pass

    def command(
        self,
        action: str,
        resource: Optional[str] = None,
        app: Optional[str] = None,
        datastore_id: Optional[UUID] = None,
        kwargs: Optional[dict] = None,
        process_states: Optional[list[str]] = None,
        process_types: Optional[list[str]] = None,
    ):
        return self._command(
            action=action,
            resource=resource,
            app=app,
            datastore_id=datastore_id,
            kwargs=kwargs,
            process_states=process_states,
            process_types=process_types,
        )

    def _command_by_model(self, data: models.CommandCreateRequest):
        return self.command(**data.model_dump())



from typing import Optional
from uuid import UUID

from pydantic import BaseModel

from lqs.interface.dsm.models.__common__ import (
    DataResponseModel,
    PaginationModel,
)


class Inference(BaseModel):
    output: dict | list
    meta: dict


class InferenceDataResponse(DataResponseModel[Inference]):
    pass


class InferenceListResponse(PaginationModel[Inference]):
    pass


class InferenceCreateRequest(BaseModel):
    datastore_id: UUID
    topic_id: UUID
    timestamp: int
    pipeline_task: str
    pipeline_model: Optional[str] = None
    pipeline_revision: Optional[str] = None


class InferenceUpdateRequest(BaseModel):
    pass



import os
import json
import base64
from textwrap import dedent
from typing import Optional

import uvicorn
from mangum import Mangum
from fastapi import FastAPI, status, Request
from fastapi.responses import JSONResponse
from fastapi.exceptions import HTTPException

from lqs.interface.dsm.models import EventCreateRequest


def validate(headers: dict, expected_secret: Optional[str] = None):
    """
    Validate the authorization header.

    If expected_secret is None, then no validation is performed.
    Otherwise, the authorization header must be present and must
    match the expected_secret.

    The authorization header must be of the form:

        Bearer <base64 encoded secret>

    The base64 encoded secret must be the expected_secret.
    """
    if expected_secret is None:
        return True
    token = headers.get("authorization")
    if token is None:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Authorization header is missing",
        )
    assert type(token) is str, "Authorization header must be a string"
    assert token.startswith("Bearer "), "Authorization header must start with 'Bearer '"
    encoded_secret = token.split(" ")[1]
    received_secret = base64.b64decode(encoded_secret).decode("utf-8")
    if received_secret != expected_secret:
        raise HTTPException(
            status_code=status.HTTP_401_UNAUTHORIZED,
            detail="Invalid authorization token",
        )
    return True


def get_server(
    event_callback: callable,
    pre_validate_callback: callable = None,
    expected_secret: str = None,
    path: str = "/events",
):
    """
    Get a FastAPI server that can be run with uvicorn, Mangum, etc.
    """
    if expected_secret is None:
        expected_secret = os.environ.get("LQS_EXPECTED_SECRET")

    description = dedent(
        """
    A small server that listens for workflow hook events from LogQS and forwards them to a callback function.

    This server has a single endpoint, /events, which only accepts POST requests.
    The body of the request must be a JSON object that matches the EventCreateRequest model.
    This is what can be expected from a LogQS workflow hook event.

    If configured, the server will validate the authorization header. The authorization header must be of the form:

        Bearer <base64 encoded secret>

    The base64 encoded secret must be the expected_secret.
    """
    )

    app = FastAPI(
        title="LogQS Event Listener", description=description, version="1.0.0"
    )

    @app.post(path, name="Receive Event", tags=["Event"], operation_id="receive_event")
    async def receive_event(event: EventCreateRequest, request: Request):
        headers = dict(request.headers)
        if pre_validate_callback is not None:
            headers = pre_validate_callback(headers)
        validate(headers, expected_secret)
        return event_callback(event)

    @app.exception_handler(Exception)
    async def exception_handler(request: Request, exc: Exception):
        return JSONResponse(
            status_code=500,
            content={"message": f"[InternalServerError] {exc}"},
        )

    return app


def run(
    event_callback: callable,
    pre_validate_callback: callable = None,
    expected_secret: str = None,
    host: str = "0.0.0.0",
    port: int = 80,
    path: str = "/events",
):
    server = get_server(
        event_callback=event_callback,
        pre_validate_callback=pre_validate_callback,
        expected_secret=expected_secret,
        path=path,
    )
    uvicorn.run(server, host=host, port=port)


def lambda_handler_handler(
    event,
    context,
    event_callback: callable,
    pre_validate_callback: callable = None,
    expected_secret: str = None,
    path: str = "/events",
):
    try:
        server = get_server(
            event_callback=event_callback,
            pre_validate_callback=pre_validate_callback,
            expected_secret=expected_secret,
            path=path,
        )
        handler = Mangum(server)
        response = handler(event, context)
        return response
    except Exception as e:
        return {
            "statusCode": 500,
            "body": json.dumps({"status": "error", "message": str(e)}),
        }


if __name__ == "__main__":

    def event_callback(**params):
        print(params)
        return {"status": "ok"}

    run(event_callback=event_callback, host="localhost", port=8081)



from lqs.interface.admin import CreateInterface
from lqs.client.common import RESTInterface
import lqs.interface.admin.models as models


class Create(CreateInterface, RESTInterface):
    service: str = "admin"

    def __init__(self, app):
        super().__init__(app=app)

    def _command(self, **params):
        return self._create_resource("commands", params, models.CommandDataResponse)



from uuid import UUID
from typing import Optional

from pydantic import BaseModel

from lqs.interface.dsm.models.__common__ import (
    CommonModel,
    DataResponseModel,
    PaginationModel,
    optional_field,
)


class DataStore(CommonModel):
    name: str
    note: Optional[str]
    context: Optional[dict]
    owner_id: Optional[UUID]
    config: Optional[dict]
    version: Optional[str]
    region: Optional[str]
    endpoint: Optional[str]
    disabled: bool
    plan: Optional[str]
    record_count: int


class DataStoreInternal(DataStore):
    stripe_subscription_id: Optional[str]


class DataStoreDataResponse(DataResponseModel[DataStore]):
    pass


class DataStoreListResponse(PaginationModel[DataStore]):
    pass


class DataStoreCreateRequest(BaseModel):
    name: str
    note: Optional[str] = None
    context: Optional[dict] = None
    owner_id: Optional[UUID] = None
    config: Optional[dict] = None
    version: Optional[str] = None
    region: Optional[str] = None
    endpoint: Optional[str] = None
    disabled: bool = False


class DataStoreUpdateRequest(BaseModel):
    name: str = optional_field
    note: Optional[str] = optional_field
    context: Optional[dict] = optional_field
    owner_id: Optional[UUID] = optional_field
    config: Optional[dict] = optional_field
    version: Optional[str] = optional_field
    region: Optional[str] = optional_field
    endpoint: Optional[str] = optional_field
    disabled: bool = optional_field



from uuid import UUID

from lqs.interface.dsm import DeleteInterface
from lqs.client.common import RESTInterface

# TODO: make this consistent with other interfaces


class Delete(DeleteInterface, RESTInterface):
    service: str = "dsm"

    def __init__(self, app):
        super().__init__(app=app)

    def _announcement(self, announcement_id: UUID):
        self._delete_resource(f"announcements/{announcement_id}")
        return

    def _api_key(self, api_key_id: UUID):
        self._delete_resource(f"apiKeys/{api_key_id}")
        return

    def _comment(self, comment_id: UUID):
        self._delete_resource(f"comments/{comment_id}")
        return

    def _configuration(self, configuration_id: UUID):
        self._delete_resource(f"configurations/{configuration_id}")
        return

    def _datastore(self, datastore_id: UUID):
        self._delete_resource(f"dataStores/{datastore_id}")
        return

    def _datastore_association(self, datastore_association_id: UUID):
        self._delete_resource(f"dataStoreAssociations/{datastore_association_id}")
        return

    def _event(self, event_id: UUID):
        self._delete_resource(f"events/{event_id}")
        return

    def _job(self, job_id: UUID):
        self._delete_resource(f"jobs/{job_id}")
        return

    def _role(self, role_id: UUID):
        self._delete_resource(f"roles/{role_id}")
        return

    def _user(self, user_id: UUID):
        self._delete_resource(f"users/{user_id}")
        return
